"""
    domonic.i18n.it
    ====================================

    html tags in Italian

    first translated to Italian using google translate.
    send a pull request if you have a better translation.
"""
from domonic.html import *

corpo = body
testa = head
# script = script
stile = style
portale = portal
forte = strong
citazione = blockquote
tabella = table
forma = form
etichetta = label
inviare = submit
titolo = title
noscript = noscript
sezione = section
# nav = nav
articolo = article
aparte = aside
hgruppo = hgroup
indirizzo = address
figura = figure
figcaption = figcaption
piccolo = small
citare = cite
codice = code
sotto = sub
# sup = sup
segno = mark
rubino = ruby
# span = span
# iframe = iframe
video = video
audio = audio
tela = canvas
didascalia = caption
colgroup = colgroup
# tbody = tbody
# testa = thead
tpiede = tfoot
setdicampi = fieldset
legenda = legend
pulsante = button
seleziona = select
listadati = datalist
optgroup = optgroup
opzione = option
areaditesto = textarea
uscita = output
progresso = progress
metro = meter
dettagli = details
riassunto = summary
menù = menu
vocedelmenu = menuitem
carattere = font
intestazione = header
pièdipagina = footer
# base = base
collegamento = link
# meta = meta
# img = img
incorporare = embed
fonte = source
traccia = track
# area = area
# col = col
ingresso = input
comando = command
principale = main
caratteredibase = basefont
centro = center
incorporare = embed
carattere = font
isindice = isindex
lista = listing
testoinchiaro = plaintext
sciopero = strike
modello = template
immagine = picture
# doctype = doctype
commento = comment
