"""
    domonic.i18n.hi
    ====================================

    html tags in Hindi

    first translated to Hindi using google translate.
    send a pull request if you have a better translation.
"""
from domonic.html import *

शरीर = shareer = body
सिर = sir = head
लिपि = lipi = script
शैली = shailee = style
पोर्टल = portal = portal
मजबूत = majaboot = strong
ब्लॉककोट = blokakot = blockquote
टेबल = tebal = table
फॉर्म = phorm = form
लेबल = lebal = label
सबमिटकरें = sabamit = submit
शीर्षक = sheershak = title
नोस्क्रिप्ट = noskript = noscript
खंड = khand = section
नव = nav = nav
लेख = lekh = article
एकतरफ = ek = aside
एचग्रुप = echagrup = hgroup
पता = pata = address
आकृति = aakrti = figure
अंजीर = anjeer = figcaption
छोटा = chhota = small
उद्धरण = uddharan = cite
कोड = kod = code
उप = up = sub
समर्थन = samarthan = sup
निशान = nishaan = mark
माणिक = maanik = ruby
अवधि = avadhi = span
आईफ्रेम = aaeephrem = iframe
वीडियो = veediyo = video
ऑडियो = odiyo = audio
कैनवास = kainavaas = canvas
कैप्शन = kaipshan = caption
कोलग्रुप = kolagrup = colgroup
शरीर = shareer = tbody
थैड = thaid = thead
टफ़ुट = tafut = tfoot
फ़ील्डसेट = feeldaset = fieldset
किंवदंती = kinvadantee = legend
बटन = batan = button
चुनें = chunen = select
डेटालिस्ट = detaalist = datalist
ऑप्टग्रुप = optagrup = optgroup
विकल्प = vikalp = option
टेक्सटेरिया = teksateriya = textarea
आउटपुट = aautaput = output
प्रगति = pragati = progress
मीटर = meetar = meter
विवरण = vivaran = details
सारांश = saaraansh = summary
मेनू = menoo = menu
मेन्युइटम = menyuitam = menuitem
फ़ॉन्ट = font = font
हेडर = hedar = header
पादलेख = paad = footer
आधार = aadhaar = base
लिंक = link = link
मेटा = meta = meta
आईएमजी = aaeeemajee = img
एम्बेड = embed = embed
स्रोत = srot = source
ट्रैक = traik = track
क्षेत्र = kshetr = area
कर्नल = karnal = col
इनपुट = inaput = input
आदेश = aadesh = command
मुख्य = mukhy = main
बेसफॉन्ट = besaphont = basefont
केंद्र = kendr = center
एम्बेड = embed = embed
फ़ॉन्ट = font = font
आइसइंडेक्स = aaisindeks = isindex
लिस्टिंग = listing = listing
सादापाठ = saada = plaintext
हड़ताल = hadataal = strike
टेम्पलेट = tempalet = template
चित्र = chitr = picture
सिद्धांत = siddhaant = doctype
टिप्पणी = tippanee = comment
